#!/usr/bin/perl
 
   use strict;
   use warnings;
   use Cwd 'abs_path';
   use POSIX;
   use LWP::UserAgent;
 
   # (c)2019 Comweb NL All RIGHTS RESERVED
   # RJHMM van den Bergh sales2@comweb.nl
   # use at own risk, no liability

  # usage cd to/Your/HTML/root/direcory/on/your/win32/system
  # perl makeXML.pl > myXMLmap.xml
  
  # EDIT THIS ONE IF NEEDED
  my $prefix="https://www.comweb.nl";

  print <<HEADER;
  
  <urlset
      xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"
      xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 
         http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">
           
  <!-- Created by Perl script provided by Comweb Netherlands -->
  <!-- (c)Comweb.nl RJHM van den Bergh -->
  <!-- Free for non commercial use -->
            
HEADER
  
  my $startlocationDirectory=abs_path(".");
  my $level=0; # directory level  
 
  getdir($startlocationDirectory); 
  
  sub getdir
  {
      my $dir = shift;
      my $path="";
      opendir (my $dh, $dir) || die "Cannot opend $dir $!";
 
      while( my $file = readdir($dh)) 
      { 
          next if ($file =~ m[^\.{1,2}$]);   # Ignore . and ..
          if (-d $file)  # is directory?
          {
              # directory found add a comment in the sitemap for readability
              print "<!-- DIR $prefix/$file -->\n";
              
              # go one directory level up
              $path = $dir .'/'. $file;  # no $file here is a directory name
              chdir($path); $level++;    # change working directory 
              $path=getcwd();            # get working directory
              $path =~ s/\\/\//g ;       # change \ (w32) to / (Internet url)

              getdir($path);             # recursive call of this function
            
              # getting out of the directory 
              # so go level down
              chdir("..");  $level--;    # change working directory
              $path=getcwd();            # get working directory  
              $path =~ s/\\/\//g ;       # change \ (w32) to / (Internet url)   
        
          } elsif (-f $file)   #no directory perhaps is a file?
          {
              # it is not a directory but a file
              # only look for certain types of extensions           
              if ($file =~ m\(htmL|htm|txt|shtml|zip|java|php|jpg|png|jpeg|gif|avi|pdf|mpg|mpeg)$\i)  
              {
                  # print the <url>   (mandantory)
                  print "<url>\n";

                  # calculate Internet location url
                  my $filepath=getcwd();     # get working directory
                  $filepath =~ s/\\/\//g ;   # change \ (w32) to / (Internet url)
                  # need to replace first part of path\.... with htt//...... ($prefix)
                  my $location = $filepath;
                  $location =~ s/$startlocationDirectory/$prefix/g;
                  $location .="/$file";      # not forget to append the filename

                  # some characters are not allowed in urls like & < > and whitespace
                  my $replace="&amp;";
                  $location =~ s/\&/$replace/g ;  #subtitute & with & 
                  $replace="%20";
                  $location =~ s/ /$replace/g ;       #substute whitespace with %20        

                  # print the <loc>htt//....myurl</loc>   (mandantory)
                  print "  <loc>$location</loc>\n";
                  
                  printLastmod($location);

                  # calculate a priority 
                  my $priority=ceil((1*0.9**$level)*100)/100;
                  # print the <priority>x.x</priority> tag 
                  print "  <priority>$priority</priority>\n";
                
                  # close the <url> tag 
                  print "</url>\n";
              } 
         }        
     }
     closedir ($dh) || die "Failed to close directo $!";
 }

 # print footer with closing </urlset>
 print <<FOOTER;
 
 </urlset>
 
 <!-- Created by Perl script provided by Comweb Netherlands -->
 <!-- (c)2019 Comweb.nl RJHM van den Bergh -->
 <!-- Free for non commercial use -->
 <!-- use at own risk, beta version -->
          
FOOTER

 sub printLastmod
 {
    my $location = shift;
    my $agent = new LWP::UserAgent;
    my $response = $agent->head($location); # sometimes hangs for 2 minutes
    #https://stackoverflow.com/questions/12391671/perl-lwpuseragent-hangs-for-120-seconds-seemingly-randomly-against-a-given-ser
    my $lastmod=($response->last_modified);
    $lastmod //= 0; # if $lastmod set to undef (in line above) then set it to 0   
    if ( $lastmod!=0) # if 0 probably could not get $lastmod  (for example 404 page)
    {
      my $w3ctime=&w3ctime($lastmod);
      print  "  <lastmod>$w3ctime</lastmod>\n";
    } else
    {
      print  "  <!-- WARNING could not get last modified from Internet 404? -->\n"; 
    }
 }
 
sub w3ctime()
{
    my $time_passed_since_epoch=shift;  #  time in seconds since 1 jan 1970 00:00:00
    # initialize some variables
    my $year=1970;
    my $isLeapYear=0;
    my $leapYearInSeconds=366*24*60*60;
    my $yearInSeconds    =365*24*60*60;
    my $start_counter=0;
    
    
    while( $start_counter<=$time_passed_since_epoch)
    {
      #determine if it is a leap year      
      if (($year % 4)==0   ) { $isLeapYear=1; } else { $isLeapYear=0; }
      if (($year % 100)==0 ) { $isLeapYear=0; } # exception
      if (($year % 400)==0 ) { $isLeapYear=1; } # exception      
      
      # add seconds of that year to the start counter
      if( $isLeapYear ) 
      {
         $start_counter+=$leapYearInSeconds; 
      } else
      {
         $start_counter+=$yearInSeconds;
      }      
      $year++;      
    }
    # $tart passed $time_passed
    $year--;
    if( $isLeapYear ) 
    {
      $start_counter-=$leapYearInSeconds; 
    } else
    {
      $start_counter-=$yearInSeconds;
    }
    
    my $month=0;
    # can add jan 31 days?
    if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
    {
       $start_counter+=31*24*60*60;  
       $month++;

       # can add februari?
       my $februariDays=28;
       if ($isLeapYear) { $februariDays=29;}
       if ( ($start_counter+$februariDays*24*60*60)<=$time_passed_since_epoch )
       {
          $start_counter+=$februariDays*24*60*60;  
          $month++;
       
          # can add march?
          if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
          {
             $start_counter+=31*24*60*60;  
             $month++;
       
             # can add april?
             if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
             {
                $start_counter+=30*24*60*60;  
                $month++;

                # can add may?
                if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                {
                   $start_counter+=31*24*60*60;  
                   $month++;
                
                   # can add june?
                   if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                   {
                      $start_counter+=30*24*60*60;  
                      $month++;

                      # can add july?
                      if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                      {
                         $start_counter+=31*24*60*60;  
                         $month++;

                         # can add august?
                         if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                         {
                            $start_counter+=31*24*60*60;  
                            $month++;

                            # can add september?
                            if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                            {
                               $start_counter+=30*24*60*60;  
                               $month++;

                               # can add oktober?
                               if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                               {
                                  $start_counter+=31*24*60*60;  
                                  $month++;
                               
                                  # can add november?
                                  if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                                  {
                                     $start_counter+=30*24*60*60;  
                                     $month++;

                                     # can add december?
                                     if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                                     {
                                        $start_counter+=31*24*60*60;  
                                        $month++;

                                        # if it comes here then something not correct
                                        # should not be possible to add full decembre month
                                        # because then it is next year
                                        die "Oops it shouldn't get here";
                                                                                                                     
                                     }
                                  }
                               }
                            }
                         }
                      }
                   }
                }
             }
          }
       }
    }
    
    # calculate days in the month lowest day = 0
    my $days=floor(($time_passed_since_epoch-$start_counter)/(24*60*60));
    $start_counter+=($days*24*60*60);
    if (($days+1)>31) { die "Oops days can't be bigger than 31.";}
    
    #calculate hours lowest = 0
    my $hours=floor(($time_passed_since_epoch-$start_counter)/(60*60));
    $start_counter+=($hours*60*60);
    if ($hours>=24) { die "Oops hours must be less than 24.";}
    
    # calculate minutes lowest = 0
    my $minutes=floor(($time_passed_since_epoch-$start_counter)/(60));
    $start_counter+=($minutes*60);
    if ($minutes>=60) { die "Oops minutes must be less than 60.";}

    my $seconds=floor($time_passed_since_epoch-$start_counter);
    if ($seconds>=60) { die "Oops seconds must be less than 60.";}
    
    # putting everything in the right format
    $year  = sprintf("%04d", $year); 
    $month = sprintf("%02d", $month+1);
    $days  = sprintf("%02d", $days+1); 
    $hours   = sprintf("%02d", $hours);  
    $minutes = sprintf("%02d", $minutes); 
    $seconds = sprintf("%02d", $seconds);
    
    my $w3cTimeString="$year-".($month)."-".($days)."T".$hours.":".$minutes.":".$seconds."Z";
    return $w3cTimeString; 
}
